#ifdef WIN32
  #define _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING
  #define _SILENCE_ALL_CXX17_DEPRECATION_WARNINGS
  #pragma warning(disable:4996)
  #include <windows.h>
#else
  #include <unistd.h>
#endif

#include <codecvt>
#include <cstring>
#include <climits>
#include <filesystem>

#include "system.hpp"

namespace fcf {
  std::string get_system_error_str(int a_errno) {
    #ifdef WIN32
      std::string result;
      char* buffer;
      DWORD len = FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                                 NULL, a_errno, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)&buffer, 0, NULL);
      if (len) {
        result.append(buffer, buffer + len);
        LocalFree(buffer);
      }
      return result;
    #else
      return strerror(a_errno);
    #endif
  }

  std::string get_exe_directory(){
    #ifndef WIN32
      char result[PATH_MAX];
      ssize_t count = readlink( "/proc/self/exe", result, PATH_MAX );
      std::filesystem::path path(std::string( result, (count > 0) ? count : 0 ));
      return std::string((const char*)path.parent_path().u8string().c_str());
    #else
      wchar_t buffer[MAX_PATH];
      GetModuleFileNameW(NULL, buffer, MAX_PATH);
      std::filesystem::path path(buffer);
      std::u16string u16str(path.parent_path().u16string());
      return std::string((const char*)std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t>().to_bytes(u16str).c_str());
    #endif
  }

}